@@ -1,10 +1,10 @@
 ;;; nov.el --- Featureful EPUB reader mode
 
-;; Copyright (C) 2017 Vasilij Schneidermann <mail@vasilij.de>
+;; Copyright (C) 2017-2018 Vasilij Schneidermann <mail@vasilij.de>
 
 ;; Author: Vasilij Schneidermann <mail@vasilij.de>
 ;; URL: https://github.com/wasamasa/nov.el
-;; Version: 0.2.2
+;; Version: 0.2.4
 ;; Package-Requires: ((dash "2.12.0") (esxml "0.3.3") (emacs "24.4"))
 ;; Keywords: hypermedia, multimedia, epub
 
@@ -99,11 +99,14 @@ If set to `nil', no saving and restoring is performed."
                  (const :tag "Don't save last reading places" nil))
   :group 'nov)
 
+(defvar-local nov-file-name nil
+  "Path to the EPUB file backing this buffer.")
+
 (defvar-local nov-temp-dir nil
   "Temporary directory containing the buffer's EPUB files.")
 
 (defvar-local nov-content-file nil
-  "Path of the EPUB buffer's .opf file.")
+  "Path to the EPUB buffer's .opf file.")
 
 (defvar-local nov-epub-version nil
   "Version string of the EPUB buffer.")
@@ -225,9 +228,9 @@ If PARSE-XML-P is t, return the contents as parsed by libxml."
     version))
 
 (defun nov-content-unique-identifier-name (content)
-  "Return the UUID name for CONTENT.
+  "Return the unique identifier name referenced in CONTENT.
 This is used in `nov-content-unique-identifier' to retrieve the
-UUID."
+the specific type of unique identifier."
   (let* ((node (esxml-query "package[unique-identifier]" content))
          (name (esxml-node-attribute 'unique-identifier node)))
     (when (not name)
@@ -235,7 +238,7 @@ UUID."
     name))
 
 (defun nov-content-unique-identifier (content)
-  "Return the UUID for CONTENT."
+  "Return the the unique identifier for CONTENT."
   (let* ((name (nov-content-unique-identifier-name content))
          (selector (format "package>metadata>identifier[id='%s']"
                            (regexp-quote name)))
@@ -650,6 +653,11 @@ Saving is only done if `nov-save-place-file' is set."
       (with-temp-file nov-save-place-file
         (insert (prin1-to-string places))))))
 
+(defun nov--index-valid-p (documents index)
+  (and (integerp index)
+       (>= index 0)
+       (< index (length documents))))
+
 ;;;###autoload
 (define-derived-mode nov-mode special-mode "EPUB"
   "Major mode for reading EPUB documents"
@@ -681,14 +689,19 @@ Saving is only done if `nov-save-place-file' is set."
                              (apply 'vector)))
     (setq nov-documents-index 0))
   (setq buffer-undo-list t)
+  (setq nov-file-name (buffer-file-name))
   (set-visited-file-name nil t) ; disable autosaves and save questions
   (let ((place (nov-saved-place (cdr (assq 'identifier nov-metadata)))))
     (if place
         (let ((index (cdr (assq 'index place)))
               (point (cdr (assq 'point place))))
-          (setq nov-documents-index index)
-          (nov-render-document)
-          (goto-char point))
+          (if (nov--index-valid-p nov-documents index)
+              (progn
+                (setq nov-documents-index index)
+                (nov-render-document)
+                (goto-char point))
+            (warn "Couldn't restore last position")
+            (nov-render-document)))
       (nov-render-document))))
 
 (provide 'nov)
